/*
 * Program: wwl
 * 
 * Description: Given two Maidenhead locators, 
 *              calculates distance and azimuth.
 *
 * Revision: 1.2  2000/11/03
 * + fixed a compile warning telling main() was not
 *   of type int, infact it was void!
 * + added a Makefile
 * 
 * Revision: 1.1  1998/05/22
 * First release
 *
 * (C) 1998-2000 by IK0ZSN Mirko Caserta <ik0zsn@amsat.org>
 * This software is placed under the terms of the 
 * GNU General Public Licence that you may find at
 * http://www.gnu.org/
 *
 * Warning: this short program has been my first attempt in
 *          C programming. It is a translation in C of an original
 *          BASIC software written by I0FLY. My software shows most
 *          of C programming errors such as not commenting anything
 *          and using not-meaningful variable names. Never program
 *          like this! This program could have been written much better
 *          than I did but I have no time to fix it now. You are welcome
 *          to do it and send me patches anyhow.
 */
/*
 * This version by va3db db@db.net db@FreeBSD.org September 28 2007
 * diffs were greater than the original.
 *
 */ 
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <ctype.h>

#define SCRATCH_WWL_LEN 9
#define WWL_LEN		6

int is_valid_locator(const char wwl[]);
void convert_locator(char *wwl, double *t, double *s);
void upstring(char *s);

int main (int argc, char **argv) {
  int l, p;
  double t, s, v, u, c, d, e, lx;
  char my_wwl[SCRATCH_WWL_LEN], dx_wwl[SCRATCH_WWL_LEN];

  printf("wwl 1.2 (va3db) - given two maidenhead locators,\n");
  printf("          calculates distance (qrb) and azimuth.\n\n");

  if (argc != 3) {
    printf("Usage: wwl my_ww_locator dx_ww_locator\n");
    return(1); /* return error */
  }
  snprintf(my_wwl, sizeof(my_wwl), "%sAA", argv[1]);
  my_wwl[WWL_LEN] ='\0';
  snprintf(dx_wwl, sizeof(dx_wwl), "%sAA", argv[2]);
  dx_wwl[WWL_LEN] ='\0';
  if (strlen(my_wwl) != WWL_LEN) {
    printf("%s: not a valid locator\n", argv[1]);
    return(1); /* return error */
  }
	
  if (strlen(dx_wwl) != WWL_LEN) {
    printf("%s: not a valid locator\n", argv[2]);
    return(1); /* return error */
  }	
 
  convert_locator(my_wwl,&t,&s);
  if(!is_valid_locator(my_wwl)) { 
    printf("%s: not a valid locator\n", my_wwl);
    return(1); /* return error */

  }

  convert_locator(dx_wwl,&v, &u);
  if (!is_valid_locator(dx_wwl)) {
    printf("%s: not a valid locator\n", dx_wwl);
    return(1); /* return error */
  }

  c = cos(s) * cos(u) * cos(v - t) + sin(s) * sin(u);
  d = 1 - c * c;
  if (sqrt(d) != 0) {
    e = (sin(u) - sin(s) * c) / (cos(s) * sqrt(d));
    if (e > 1.0)
      e = 1.0;
    else if (e < -1.0)
      e = -1.0;
    lx = acos(e) * (180 / M_PI);
    l = (int) (lx + 0.5);
  } else { l = 0; }
	
  if ((v - t) < 0) { l = 360 - l; }
	
  p = (int) ((atan(sqrt(d) / c) * 6371.33) + .5);
  printf("qrb: %d kilometers, azimuth: %d degrees\n", p, l);

  return(0); /* return successful */
	
}

/*
 * is_valid_locator
 * check for valid locator
 *
 * inputs	- string to locator
 * output	- 1 if valid locator 0 if not
 * side effects	-
 */

int
is_valid_locator(const char wwl[])
{
  if (wwl[0] < 'A' || wwl[0] > 'X' ||
      wwl[1] < 'A' || wwl[1] > 'X' ||
      wwl[2] < '0' || wwl[2] > '9' ||
      wwl[3] < '0' || wwl[3] > '9' ||
      wwl[4] < 'A' || wwl[4] > 'X' ||
      wwl[5] < 'A' || wwl[5] > 'X' ) 
	return 0;
  else
	return 1;
}

/*
 * convert_locator
 * upper case locator and return doubles for each field in locator.
 *
 * inputs	- string to convert
 *		- pointers to each double to return as converted
 * side effects	-
 * bugs		- could be refactored better
 */

void
convert_locator(char *wwl, double *t, double *s)
{
  double z, y, n, h, x, w;
  upstring(wwl);
  z = (double)(wwl[0] - 'A');
  y = (double)(wwl[1] - 'A');
  n = (double)(wwl[2] - '0');
  h = (double)(wwl[3] - '0');
  x = (double)(wwl[4] - 'A');
  w = (double)(wwl[5] - 'A');
  *t = z * 20 - 180 + n * 2 + x / 12 + 1 / 24;
  *t = *t * (M_PI / 180); 
  *s = y * 10 - 90 + h + w / 24 + 1 / 48;
  *s = *s * (M_PI / 180);
}

/*
 * upstring
 * convert string to upper case
 *
 * inputs	- string to convert to upper case
 * output	- none
 * side effects	- string modified in place
 */
void
upstring(char *s)
{
	while(*s != '\0') {
		*s = toupper(*s);
		s++;
	}
}
